/*
 * tlsproxyd.h:
 * Definitions for tlsproxyd.
 *
 * Copyright (c) 2002 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 * $Id: tlsproxyd.h,v 1.3 2002/07/05 14:19:50 chris Exp $
 *
 */

#ifndef __TLSPROXYD_H_ /* include guard */
#define __TLSPROXYD_H_

#include <sys/types.h>

#include <arpa/inet.h>

#include <netinet/in.h>

#include <openssl/err.h>
#include <openssl/ssl.h>

#include <sys/socket.h>

#define PROGNAME    "tlsproxyd"

#ifndef CFGFILE
#   define CFGFILE  "/etc/tlsproxyd.conf"
#endif

/* struct config:
 * Configuration directives. */
struct config {
    char *certfile, *pkeyfile;
    char *pidfile;
    int logfacil;
    uid_t uid;
    gid_t gid;
    int maxprocs;
    int timeout;
    int nproxy;
    struct proxyent *proxy;
};

/* struct proxyent:
 * Definition of a service proxy. */
struct proxyent {
    struct sockaddr_in from, to;
    int s;
};

/* util.c */
int daemon(int nochdir, int noclose);
void *xmalloc(size_t n);
void *xcalloc(size_t n, size_t m);
void *xrealloc(void *w, size_t n);
char *xstrdup(const char *s);
void xfree(void *v);
ssize_t xread(int fd, void *buf, size_t count);
ssize_t xwrite(int fd, const void *buf, size_t count);
void (*xsignal(int signum, void (*handler)(int)))(int);
int strtoaddr(const char *s, struct in_addr *ad);
int strtoport(const char *s, unsigned short *port);
int strtouid(const char *user, uid_t *u);
gid_t strtogid(const char *group, gid_t *g);
void log_print(int priority, const char *fmt, ...);
int fcntl_lock(int fd);
int fcntl_unlock(int fd);

/* config.c */
struct config *read_config(const char *filename);

/* proxy.c */
int do_proxy(int s_from, const struct sockaddr_in *a_to, struct config *cfg, SSL_CTX *ctx);
int do_proxy_accept(struct config *cfg, SSL_CTX *ctx);

/* block_signal:
 * Block a signal using sigprocmask. */
#define block_signal(s)     do {                                        \
                                sigset_t set;                           \
                                sigemptyset(&set);                      \
                                sigaddset(&set, s);                     \
                                sigprocmask(SIG_BLOCK, &set, NULL);     \
                            } while (0)

/* unblock_signal:
 * Unblock a signal using sigprocmask. */
#define unblock_signal(s)   do {                                        \
                                sigset_t set;                           \
                                sigemptyset(&set);                      \
                                sigaddset(&set, s);                     \
                                sigprocmask(SIG_UNBLOCK, &set, NULL);   \
                            } while (0)


#endif /* __TLSPROXYD_H_ */
